typedef struct{
	int nbVar, nbVal, valMin;
	int* nbConstraints;   // nbConstraints[Xi] = nb of constraints involving Xi
	int** constrainedVar; // constrainedVar[Xi][j] = jth variable involved in a constraint with Xi
	int*** nbNoGoods;     // nbNoGoods[Xi][Xj][Vk] = nb of values v so that (Vk,v) is a no good for (Xi,Xj)
	int**** noGood;       // noGood[Xi][Xj][Vk][l] = lth value v so that (Vk,v) is a no good for (Xi,Xj)
	int**** isConsistent; // isConsistent[Xi][Xj][Vi][Vj] = 1 if {Xi=Vi,Xj=Vj} is consistent ; 0 otherwise
} Tcsp;



int checkSolution(Tcsp csp, int* solution){
	// Returns the number of violated constraints in solution
	int i, j;
	int cpt = 0;
	for (i=0; i<csp.nbVar; i++)
		for (j=i+1; j<csp.nbVar; j++)
			cpt += 1-csp.isConsistent[i][solution[i]][j][solution[j]];
	return cpt;
}

int printSolution(Tcsp csp, int* solution){
	// displays solution
	int i;
	printf("Best solution found is:\n");
	for (i=0; i<csp.nbVar; i++)
		printf("X%d=%d ",i,solution[i]);
	printf("\n");
	return 0;
}

void getCSP(Tcsp *csp, char* fileName){
	// initialize csp with data contained in fileName
	FILE* fd;
	int var1, var2, val1, val2, i, j, k, l;
	
	fd=fopen(fileName, "r");
	if (fd == NULL){
		printf("Error while opening file %s\n",fileName);
		exit(1);
	}
	fscanf(fd,"%d %d", &(csp->nbVar), &(csp->nbVal));
	csp->nbConstraints = (int*)calloc(csp->nbVar,sizeof(int));
	csp->constrainedVar = (int**)calloc(csp->nbVar,sizeof(int*));
	csp->nbNoGoods = (int***)calloc(csp->nbVar,sizeof(int**));
	csp->noGood = (int****)calloc(csp->nbVar,sizeof(int***));
	csp->isConsistent = (int****)calloc(csp->nbVar,sizeof(int***));
	for (i=0; i<csp->nbVar; i++){
		csp->constrainedVar[i] = (int*)calloc(csp->nbVar,sizeof(int));
		csp->nbNoGoods[i] = (int**)calloc(csp->nbVar,sizeof(int*));
		csp->noGood[i] = (int***)calloc(csp->nbVar,sizeof(int**));
		csp->isConsistent[i] = (int***)calloc(csp->nbVar,sizeof(int**));
		for (j=0; j<csp->nbVar; j++){
			csp->nbNoGoods[i][j] = (int*)calloc(csp->nbVal,sizeof(int));
			csp->noGood[i][j] = (int**)calloc(csp->nbVal,sizeof(int*));
			csp->isConsistent[i][j] = (int**)calloc(csp->nbVal,sizeof(int*));
			for (k=0; k<csp->nbVal; k++){
				csp->noGood[i][j][k] = (int*)calloc(csp->nbVal,sizeof(int));
				csp->isConsistent[i][j][k] = (int*)calloc(csp->nbVal,sizeof(int));
				for (l=0; l<csp->nbVal; l++) csp->isConsistent[i][j][k][l] = 1;
			}
		}
	}
	fscanf(fd,"%d",&var1);
	while (var1 >= 0){
		fscanf(fd,"%d",&var2);
		csp->constrainedVar[var1][csp->nbConstraints[var1]++] = var2;
		csp->constrainedVar[var2][csp->nbConstraints[var2]++] = var1;
		fscanf(fd,"%d",&val1);
		while (val1 >= 0){
			fscanf(fd,"%d",&val2);
			csp->isConsistent[var1][var2][val1][val2] = 0;
			csp->isConsistent[var2][var1][val2][val1] = 0;
			csp->noGood[var1][var2][val1][csp->nbNoGoods[var1][var2][val1]++] = val2;
			csp->noGood[var2][var1][val2][csp->nbNoGoods[var2][var1][val2]++] = val1;
			fscanf(fd,"%d",&val1);
		}
		fscanf(fd,"%d",&var1);
	}
	fclose(fd);
}
